﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace GridSourceCreator
{
    public class GenerateContent
    {
        #region Constants
        private const string sExtensionCSharp = ".cs";
        private const string sExtensionVB = ".vb";

        private const string sTagControl = "%%";
        private const string sTagCustomConstants = "custom constants";
        private const string sTagCustomConstructors = "custom constructors";
        private const string sTagCustomData = "custom data";
        private const string sTagCustomImports = "custom imports";
        private const string sTagCustomProperties = "custom properties";
        private const string sTagCustomRoutines = "custom routines";
        private const string sTagEndIf = "%%endif";
        private const string sTagEndInsert = "%%endinsert";
        private const string sTagEndRepeat = "%%endrepeat";
        private const string sTagIf = "%%if";
        private const string sTagIfDataReader = "datareader";
        private const string sTagIfDataSet = "dataset";
        private const string sTagIfDataTable = "datatable";
        private const string sTagIfSortField = "sortfield";
        private const string sTagInsert = "%%insert";
        private const string sTagInsertConstants = "constants";
        private const string sTagInsertConstructors = "constructors";
        private const string sTagInsertData = "data";
        private const string sTagInsertImports = "imports";
        private const string sTagInsertProperties = "properties";
        private const string sTagInsertRoutines = "routines";
        private const string sTagRepeat = "%%repeat";
        private const string sTagRepeatField = "field";
        private const string sTagRegion = "region";

        private const string sTemplateFileName = "TemplateGrid";
        #endregion

        #region Enumerations
        public enum FieldType
        {
            Boolean,
            Byte,
            Character,
            Currency,
            DateTime,
            Decimal,
            Double,
            Integer,
            Long,
            Short,
            Single,
            String
        }

        public enum Language
        {
            CSharp,
            VB,
        }
        #endregion

        #region Structs
        public struct FieldDefaultCSharp
        {
            public const string Boolean = "false";
            public const string Byte = "0";
            public const string Character = "\"\"";
            public const string Currency = "0";
            public const string DateTime = "DateTime.MinValue";
            public const string Decimal = "0";
            public const string Double = "0";
            public const string Integer = "0";
            public const string Long = "0";
            public const string Short = "0";
            public const string Single = "0";
            public const string String = "\"\"";
        }

        public struct FieldDefaultVB
        {
            public const string Boolean = "False";
            public const string Byte = "0";
            public const string Character = "\"\"";
            public const string Currency = "0";
            public const string DateTime = "DateTime.MinValue";
            public const string Decimal = "0";
            public const string Double = "0";
            public const string Integer = "0";
            public const string Long = "0";
            public const string Short = "0";
            public const string Single = "0";
            public const string String = "\"\"";
        }

        public struct FieldTypeCSharp
        {
            public const string Boolean = "bool";
            public const string Byte = "byte";
            public const string Character = "string";
            public const string Currency = "decimal";
            public const string DateTime = "datetime";
            public const string Decimal = "decimal";
            public const string Double = "double";
            public const string Integer = "int";
            public const string Long = "long";
            public const string Short = "short";
            public const string Single = "float";
            public const string String = "string";
        }

        public struct FieldTypeVB
        {
            public const string Boolean = "Boolean";
            public const string Byte = "Byte";
            public const string Character = "String";
            public const string Currency = "Decimal";
            public const string DateTime = "DateTime";
            public const string Decimal = "Decimal";
            public const string Double = "Double";
            public const string Integer = "Integer";
            public const string Long = "Long";
            public const string Short = "Short";
            public const string Single = "Single";
            public const string String = "String";
        }

        public struct FieldTypePrefixCSharp
        {
            public const string Boolean = "b";
            public const string Byte = "by";
            public const string Character = "s";
            public const string Currency = "d";
            public const string DateTime = "dt";
            public const string Decimal = "d";
            public const string Double = "d";
            public const string Integer = "i";
            public const string Long = "l";
            public const string Object = "o";
            public const string Short = "sh";
            public const string Single = "f";
            public const string String = "s";
        }

        public struct FieldTypePrefixVB
        {
            public const string Boolean = "b";
            public const string Byte = "by";
            public const string Character = "s";
            public const string Currency = "d";
            public const string DateTime = "dt";
            public const string Decimal = "d";
            public const string Double = "d";
            public const string Integer = "i";
            public const string Long = "l";
            public const string Object = "o";
            public const string Short = "sh";
            public const string Single = "f";
            public const string String = "s";
        }
        #endregion

        #region Class Structures
        public class GridSourceEntry
        {
            public enum DataAccessObject
            {
                DataReader,
                DataSet,
                DataTable,
            }

            public string sDescription = "";
            public string sProjectASPXFileName = "";
            public string sProjectClassPath = "";
            public string sProjectSolutionFileName = "";
            public DataAccessObject iDataAccessObject = DataAccessObject.DataReader;
            public Language iLanguage = Language.VB;
            public bool bUpdate = false;
        }

        public class GridCommand
        {
            public GridCommand(string sCommandName)
            {
                this.sCommandName = sCommandName;
            }
            public string sCommandName = "";
        }

        public class GridField
        {
            public bool bSortField = false;
            public FieldType iFieldDataType = FieldType.String;
            public string sFieldDataType = "";
            public string sFieldDefaultValue = "";
            public string sFieldName = "";
            public string sFieldNameApp = "";
        }

        public class GridTable
        {
            public int iObjectSourceIndex = -1;
            public string sObjectSourceID = "";
            public List<GridCommand> oCommands = new List<GridCommand>();
            public List<GridField> oFields = new List<GridField>();
        }

        public class ObjectDataSource
        {
            public string sObjectSourceID = "";
            public string sObjectSourceBeanName = "";
            public string sObjectSourceName = "";
            public string sMethodDelete = "DeleteRecord";
            public string sMethodInsert = "InsertRecord";
            public string sMethodSelect = "GetRecords";
            public string sMethodSelectCount = "GetRecordCount";
            public string sMethodUpdate = "UpdateRecord";
            public string sSortParam = "SortExpression";
        }
        #endregion

        #region Data
        private GridSourceEntry m_oEntry = null;
        private List<GridTable> m_oGridTableList = new List<GridTable>();
        private List<ObjectDataSource> m_oObjectDataSourceList = new List<ObjectDataSource>();
        private List<string> m_oErrorMsgList = new List<string>();
        private List<string> m_oCustomConstants = null;
        private List<string> m_oCustomConstructors = null;
        private List<string> m_oCustomData = null;
        private List<string> m_oCustomImports = null;
        private List<string> m_oCustomProperties = null;
        private List<string> m_oCustomRoutines = null;
        private int m_iCommandIndex = -1;
        private int m_iFieldIndex = -1;
        private int m_iGridIndex = -1;
        private int m_iObjectDataSourceIndex = -1;
        private string m_sFileExtensionClass = "";
        private string m_sFileNameTemplate = "";
        private string m_sPathApp = "";
        private string m_sPathClass = "";
        #endregion

        #region Constructor
        public GenerateContent(
            string sPathApp,
            GridSourceEntry oEntry)
        {
            m_sPathApp = sPathApp;
            m_oEntry = oEntry;
        }
        #endregion

        #region Public Routines
        /// <summary>
        /// Generate classes to handle the population and processing of grid controls
        /// </summary>
        /// <param name="iGenerateCount"></param>
        /// <returns></returns>
        public List<string> Generate(out int iGenerateCount)
        {
            iGenerateCount = 0;

            try
            {
                // Set the language and class
                m_sPathClass = m_oEntry.sProjectClassPath
                    + ((m_oEntry.sProjectClassPath.EndsWith("\\")) ? "" : "\\");
                m_sFileExtensionClass =
                    ((m_oEntry.iLanguage == Language.CSharp) ? sExtensionCSharp : sExtensionVB);
                m_sFileNameTemplate =
                    m_sPathApp + sTemplateFileName + m_sFileExtensionClass;

                if (!ProcessFileExists(m_oEntry.sProjectASPXFileName, "ASPX"))
                    return m_oErrorMsgList;
                if (!ProcessFileExists(m_sFileNameTemplate, "Template"))
                    return m_oErrorMsgList;

                string sContents = File.ReadAllText(m_oEntry.sProjectASPXFileName);
                string sContentsLC = sContents.ToLower();
                ProcessParseGrid(ref sContents, ref sContentsLC);
                ProcessParseObjectDataSource(ref sContents, ref sContentsLC);
                if (m_oGridTableList.Count < 0)
                {
                    m_oErrorMsgList.Add("There were no grid entries in the ASPX page");
                    return m_oErrorMsgList;
                }
                if (m_oObjectDataSourceList.Count <= 0)
                {
                    m_oErrorMsgList.Add("There were no object data source entries in the ASPX page");
                    return m_oErrorMsgList;
                }
                foreach (GridTable oGridTable in m_oGridTableList)
                {
                    for (int iX = 0; iX < m_oObjectDataSourceList.Count; iX++)
                    {
                        if (oGridTable.sObjectSourceID.ToLower() == m_oObjectDataSourceList[iX].sObjectSourceID.ToLower())
                        {
                            oGridTable.iObjectSourceIndex = iX;
                            break;
                        }
                    }
                }
                foreach (GridTable oGridTable in m_oGridTableList)
                {
                    if (oGridTable.iObjectSourceIndex < 0)
                    {
                        m_oErrorMsgList.Add("There was no object data source with the ID " + oGridTable.sObjectSourceID + " defined in the ASPX page");
                        return m_oErrorMsgList;
                    }
                }

                for (m_iGridIndex = 0; m_iGridIndex <  m_oGridTableList.Count; m_iGridIndex++)
                {
                    m_iFieldIndex = -1;
                    m_iObjectDataSourceIndex = m_oGridTableList[m_iGridIndex].iObjectSourceIndex;
                    ProcessGridClass(
                        m_oObjectDataSourceList[m_iObjectDataSourceIndex].sObjectSourceName,
                        m_sFileNameTemplate);
                    iGenerateCount++;
                }
                return m_oErrorMsgList;
            }
            catch (Exception oEx)
            {
                m_oErrorMsgList.Add("An error occurred while processing the ASPX file"
                    + oEx.Message);
                return m_oErrorMsgList;
            }
        }
        #endregion

        #region Private Routines
        /// <summary>
        /// Process the output grid class file
        /// </summary>
        /// <param name="sClassFileName"></param>
        /// <param name="sTemplateClassFileName"></param>
        /// <returns></returns>
        private bool ProcessGridClass(string sClassFileName, string sTemplateClassFileName)
        {
            try
            {
                string[] sLines = null;

                // Set the name of the output class file
                string sFileNameClass = m_sPathClass + sClassFileName + m_sFileExtensionClass;
                if (File.Exists(sFileNameClass))
                {
                    // If the class file already exists, read in all of the text
                    sLines = File.ReadAllLines(sFileNameClass);
                    bool bCustomImports = false;
                    bool bCustomConstants = false;
                    bool bCustomConstructors = false;
                    bool bCustomData = false;
                    bool bCustomProperties = false;
                    bool bCustomRoutines = false;
                    bool bRegion = false;
                    m_oCustomConstants = new List<string>();
                    m_oCustomConstructors = new List<string>();
                    m_oCustomData = new List<string>();
                    m_oCustomImports = new List<string>();
                    m_oCustomProperties = new List<string>();
                    m_oCustomRoutines = new List<string>();

                    // Scan the existing class file looking for the Custom sections
                    foreach (string sLine in sLines)
                    {
                        string sLineText = sLine.Trim().ToLower();
                        if (!bRegion)
                        {
                            // Found the start of a region - check the region name
                            if (sLineText.StartsWith("#") && sLineText.Contains(sTagRegion))
                            {
                                bRegion = true;
                                if (sLineText.Contains(sTagCustomImports))
                                    bCustomImports = true;
                                else if (sLineText.Contains(sTagCustomConstants))
                                    bCustomConstants = true;
                                else if (sLineText.Contains(sTagCustomConstructors))
                                    bCustomConstructors = true;
                                else if (sLineText.Contains(sTagCustomData))
                                    bCustomData = true;
                                else if (sLineText.Contains(sTagCustomProperties))
                                    bCustomProperties = true;
                                else if (sLineText.Contains(sTagCustomRoutines))
                                    bCustomRoutines = true;
                            }
                        }

                        else if (bRegion)
                        {
                            // Process lines in the region until the endregion line is reached
                            if (sLineText.StartsWith("#") && sLineText.Contains(sTagRegion))
                            {
                                bCustomImports = false;
                                bCustomConstants = false;
                                bCustomConstructors = false;
                                bCustomData = false;
                                bCustomProperties = false;
                                bCustomRoutines = false;
                                bRegion = false;
                            }
                            else if (bCustomImports)
                                m_oCustomImports.Add(sLine);
                            else if (bCustomConstants)
                                m_oCustomConstants.Add(sLine);
                            else if (bCustomConstructors)
                                m_oCustomConstructors.Add(sLine);
                            else if (bCustomData)
                                m_oCustomData.Add(sLine);
                            else if (bCustomProperties)
                                m_oCustomProperties.Add(sLine);
                            else if (bCustomRoutines)
                                m_oCustomRoutines.Add(sLine);
                        }
                    }
                }

                StringBuilder oBuffer = ProcessImport(
                    sTemplateClassFileName,
                    "Class");

                if (oBuffer == null)
                    return false;
                else
                {
                    File.WriteAllText(sFileNameClass, oBuffer.ToString());
                    return true;
                }
            }
            catch (Exception oEx)
            {
                m_oErrorMsgList.Add("An error occurred while generating code for class "
                    + sClassFileName + " - "
                    + oEx.Message);
                return false;
            }
        }

        /// <summary>
        /// Process all of the grids in an ASPX file
        /// </summary>
        /// <param name="sContents"></param>
        /// <param name="sContentsLC"></param>
        /// <returns></returns>
        private bool ProcessParseGrid(ref string sContents, ref string sContentsLC)
        {
            const string sGridMicrosoftBeginGrid = "<asp:gridview";
            const string sGridMicrosoftEndGrid = "</asp:gridview";
            const string sGridMicrosoftFieldControls = "BoundField,ButtonField,CheckBoxField,CommandField,HyperLinkField,ImageField,TemplateField";
            const string sGridMicrosoftFieldKey = "<asp:";
            const string sGridTelerikBeginGrid = "<telerik:radgrid";
            const string sGridTelerikEndGrid = "</telerik:radgrid";
            const string sGridTelerikFieldControls = "GridAttachmentColumn,GridBinaryImageColumn,GridBoundColumn,GridButtonColumn,GridCalculatedColumn,GridCheckBoxColumn,GridClientSelectColumn,GridDateTimeColumn,GridDragDropColumn,GridDropDownColumn,GridHTMLEditorColumn,GridHyperLinkColumn,GridImageColumn,GridMaskedColumn,GridNumericColumn,GridRatingColumn,GridTemplateColumn";
            const string sGridTelerikFieldKey = "<telerik:";
            
            try
            {
                int iFileX = 0;
                while (true)
                {
                    // Process any Telerik grids - RadGrid controls
                    int iFileY = sContentsLC.IndexOf(sGridTelerikBeginGrid, iFileX);
                    if (iFileY < 0)
                        break;
                    int iFileZ = sContentsLC.IndexOf(sGridTelerikEndGrid, iFileY + 1);
                    if (iFileZ < 0)
                        iFileZ = sContents.Length;
                    string sText = sContents.Substring(iFileY, iFileZ - iFileY);
                    string sTextLC = sText.ToLower();

                    GridTable oGridTable = ProcessParseGridContents(sText, sTextLC, sGridTelerikFieldControls, sGridTelerikFieldKey);
                    if (oGridTable != null)
                        m_oGridTableList.Add(oGridTable);
                    iFileX = iFileZ;
                }

                iFileX = 0;
                while (true)
                {
                    // Process any Microsoft grids - GridView controls
                    int iFileY = sContentsLC.IndexOf(sGridMicrosoftBeginGrid, iFileX);
                    if (iFileY < 0)
                        break;
                    int iFileZ = sContentsLC.IndexOf(sGridMicrosoftEndGrid, iFileY + 1);
                    if (iFileZ < 0)
                        iFileZ = sContents.Length;
                    string sText = sContents.Substring(iFileY, iFileZ - iFileY);
                    string sTextLC = sText.ToLower();

                    GridTable oGridTable = ProcessParseGridContents(sText, sTextLC, sGridMicrosoftFieldControls, sGridMicrosoftFieldKey);
                    if (oGridTable != null)
                        m_oGridTableList.Add(oGridTable);
                    iFileX = iFileZ;
                }

                return true;
            }
            catch (Exception oEx)
            {
                m_oErrorMsgList.Add("An error occurred while processing an aspx page - "
                    + oEx.Message);
                return false;
            }
        }

        /// <summary>
        /// Process the columns in a grid and extract all of the fields
        /// </summary>
        /// <param name="sText"></param>
        /// <param name="sTextLC"></param>
        /// <param name="sGridFieldControls"></param>
        /// <param name="sGridFieldKey"></param>
        /// <returns></returns>
        private GridTable ProcessParseGridContents(string sText, string sTextLC, string sGridFieldControls, string sGridFieldKey)
        {
            const string sGridBeginColumns = "<Columns>";
            const string sGridCommand = "CommandName=";
            const string sGridDataField = "DataField=";
            const string sGridDataSourceID = "DataSourceID=";
            const string sGridDataTextField = "DataTextField=";
            const string sGridDataType = "DataType=";
            const string sGridEndColumns = "</Columns>";
            const string sGridSortExpression = "SortExpression=";

            try
            {
                GridTable oGridTable = new GridTable();
                string[] sFieldControlArray = sGridFieldControls.ToLower().Split(',');
                int[] iFieldControlArray = new int[sFieldControlArray.Length];
                oGridTable.sObjectSourceID = ProcessParseKey(sGridDataSourceID, 0, ref sText, ref sTextLC);
                if (oGridTable.sObjectSourceID == "")
                {
                    m_oErrorMsgList.Add("There was no " + sGridDataSourceID + " key in the grid definition");
                    return null;
                }

                int iGridX = sTextLC.IndexOf(sGridBeginColumns.ToLower());
                if (iGridX < 0)
                {
                    m_oErrorMsgList.Add("There was no " + sGridBeginColumns + " key in the grid definition");
                    return null;
                }
                int iGridY = sTextLC.IndexOf(sGridEndColumns.ToLower(), iGridX + 1);
                if (iGridX < 0)
                {
                    m_oErrorMsgList.Add("There was no " + sGridEndColumns + " key in the grid definition");
                    return null;
                }
                sText = sText.Substring(iGridX, iGridY - iGridX);
                sTextLC = sText.ToLower();

                iGridX = 0;
                while (iGridX >= 0)
                {
                    for (int iX = 0; iX < sFieldControlArray.Length; iX++)
                    {
                        iFieldControlArray[iX] = sTextLC.IndexOf(sGridFieldKey + sFieldControlArray[iX], iGridX);
                    }

                    iGridY = -1;
                    for (int iX = 0; iX < iFieldControlArray.Length; iX++)
                    {
                        if (iFieldControlArray[iX] > 0 && (iGridY < 0 || iFieldControlArray[iX] < iGridY))
                            iGridY = iFieldControlArray[iX];
                    }
                    if (iGridY > 0)
                    {
                        for (int iX = 0; iX < sFieldControlArray.Length; iX++)
                        {
                            iFieldControlArray[iX] = sTextLC.IndexOf(sGridFieldKey + sFieldControlArray[iX], iGridY + 1);
                        }
                        int iGridZ = -1;
                        for (int iX = 0; iX < iFieldControlArray.Length; iX++)
                        {
                            if (iFieldControlArray[iX] > 0 && (iGridZ < 0 || iFieldControlArray[iX] < iGridZ))
                                iGridZ = iFieldControlArray[iX];
                        }

                        string sTextSub = "";
                        if (iGridZ > iGridY)
                            sTextSub = sText.Substring(iGridY, iGridZ - iGridY);
                        else
                        {
                            sTextSub = sText.Substring(iGridY);
                            iGridZ = iGridY + sTextSub.Length;
                        }
                        string sTextSubLC = sTextSub.ToLower();

                        GridField oGridField = new GridField();
                        oGridField.sFieldName = ProcessParseKey(sGridDataField, 0, ref sTextSub, ref sTextSubLC);
                        if (oGridField.sFieldName == "")
                            oGridField.sFieldName = ProcessParseKey(sGridDataTextField, 0, ref sTextSub, ref sTextSubLC);
                        if (oGridField.sFieldName != "")
                        {
                            string sValue = ProcessParseKey(sGridSortExpression, 0, ref sTextSub, ref sTextSubLC);
                            if (sValue != "")
                                oGridField.bSortField = true;
                            sValue = ProcessParseKey(sGridDataType, 0, ref sTextSub, ref sTextSubLC);
                            ProcessParseSetDataType(oGridField, sValue);
                            oGridTable.oFields.Add(oGridField);
                        }

                        string sCommandName = ProcessParseKey(sGridCommand, 0, ref sTextSub, ref sTextSubLC);
                        if (!String.IsNullOrEmpty(sCommandName))
                        {
                            bool bFound = false;
                            for (int iX = 0; iX < oGridTable.oCommands.Count; iX++)
                            {
                                if (sCommandName.ToLower() == oGridTable.oCommands[iX].sCommandName.ToLower())
                                {
                                    bFound = true;
                                    break;
                                }
                            }
                            if (!bFound)
                                oGridTable.oCommands.Add(new GridCommand(sCommandName));
                        }
                        iGridY = iGridZ;
                    }
                    iGridX = iGridY;
                }
                return oGridTable;
            }
            catch (Exception oEx)
            {
                m_oErrorMsgList.Add("An error occurred while parsing a grid - "
                    + oEx.Message);
                return null;
            }
        }

        private string ProcessParseKey(
            string sKey,
            int iStartIndex,
            ref string sText,
            ref string sTextLC)
        {
            try
            {
                string sValue = "";
                int iX = sTextLC.IndexOf(sKey.ToLower(), iStartIndex);
                if (iX < 0)
                    return sValue;

                iX += sKey.Length;
                while (iX < sText.Length)
                {
                    if (sText.Substring(iX, 1) == " ")
                        iX++;
                    else
                        break;
                }

                if (iX < sText.Length)
                {
                    string sDelim = sText.Substring(iX, 1);
                    if (sDelim != "\"")
                        sDelim = " ";
                    int iY = sText.IndexOf(sDelim, iX + 1);
                    if (iY > iX)
                    {
                        sValue = sText.Substring(iX, iY - iX + 1);
                        if (sValue.StartsWith("\""))
                            sValue = sValue.Substring(1);
                        if (sValue.EndsWith("\""))
                            sValue = sValue.Substring(0, sValue.Length - 1);
                    }
                }
                return sValue;
            }
            catch (Exception oEx)
            {
                m_oErrorMsgList.Add("An error occurred while parsing a key - "
                    + oEx.Message);
                return "";
            }
        }

        private bool ProcessParseObjectDataSource(ref string sContents, ref string sContentsLC)
        {
            const string sObjectDataSourceBegin = "<asp:objectdatasource";
            const string sObjectDataSourceDataObjectTypeName = "DataObjectTypeName=";
            const string sObjectDataSourceDeleteMethod = "DeleteMethod=";
            const string sObjectDataSourceEndKey = "</asp:objectdatasource";
            const string sObjectDataSourceID = "ID=";
            const string sObjectDataSourceInsertMethod = "InsertMethod=";
            const string sObjectDataSourceSelectCountMethod = "SelectCountMethod=";
            const string sObjectDataSourceSelectMethod = "SelectMethod=";
            const string sObjectDataSourceSortParam = "SortParameterName=";
            const string sObjectDataSourceUpdateMethod = "UpdateMethod=";
            const string sObjectDataSourceTypeName = "TypeName=";

            try
            {
                int iFileX = 0;
                while (true)
                {
                    int iFileY = sContentsLC.IndexOf(sObjectDataSourceBegin, iFileX);
                    if (iFileY < 0)
                        break;
                    int iFileZ = sContentsLC.IndexOf(sObjectDataSourceEndKey, iFileY + 1);
                    if (iFileZ < 0)
                        iFileZ = sContents.Length;
                    string sText = sContents.Substring(iFileY, iFileZ - iFileY);
                    string sTextLC = sText.ToLower();

                    ObjectDataSource oObjectDataSource = new ObjectDataSource();
                    oObjectDataSource.sObjectSourceID = ProcessParseKey(sObjectDataSourceID, 0, ref sText, ref sTextLC);
                    if (oObjectDataSource.sObjectSourceID == "")
                    {
                        m_oErrorMsgList.Add("There was no " + sObjectDataSourceID + " key in the object data source definition");
                        return false;
                    }

                    oObjectDataSource.sObjectSourceName = ProcessParseKey(sObjectDataSourceTypeName, 0, ref sText, ref sTextLC);
                    if (oObjectDataSource.sObjectSourceName == "")
                    {
                        m_oErrorMsgList.Add("There was no " + sObjectDataSourceTypeName + " key in the object data source definition");
                        return false;
                    }

                    oObjectDataSource.sObjectSourceBeanName = ProcessParseKey(sObjectDataSourceDataObjectTypeName, 0, ref sText, ref sTextLC);
                    if (oObjectDataSource.sObjectSourceBeanName == "")
                    {
                        m_oErrorMsgList.Add("There was no " + sObjectDataSourceDataObjectTypeName + " key in the object data source definition");
                        return false;
                    }

                    string sValue = ProcessParseKey(sObjectDataSourceDeleteMethod, 0, ref sText, ref sTextLC);
                    if (sValue != "")
                        oObjectDataSource.sMethodDelete = sValue;
                    sValue = ProcessParseKey(sObjectDataSourceInsertMethod, 0, ref sText, ref sTextLC);
                    if (sValue != "")
                        oObjectDataSource.sMethodInsert = sValue;
                    sValue = ProcessParseKey(sObjectDataSourceSelectCountMethod, 0, ref sText, ref sTextLC);
                    if (sValue != "")
                        oObjectDataSource.sMethodSelectCount = sValue;
                    sValue = ProcessParseKey(sObjectDataSourceSelectMethod, 0, ref sText, ref sTextLC);
                    if (sValue != "")
                        oObjectDataSource.sMethodSelect = sValue;
                    sValue = ProcessParseKey(sObjectDataSourceUpdateMethod, 0, ref sText, ref sTextLC);
                    if (sValue != "")
                        oObjectDataSource.sMethodUpdate = sValue;
                    sValue = ProcessParseKey(sObjectDataSourceSortParam, 0, ref sText, ref sTextLC);
                    if (sValue != "")
                        oObjectDataSource.sSortParam = sValue;

                    m_oObjectDataSourceList.Add(oObjectDataSource);
                    iFileX = iFileZ;
                }
                return true;
            }
            catch (Exception oEx)
            {
                m_oErrorMsgList.Add("An error occurred while parsing an object data source - "
                    + oEx.Message);
                return false;
            }
        }

        private void ProcessParseSetDataType(GridField oGridField, string sValue)
        {
            try
            {
                if (sValue.ToLower().StartsWith("system."))
                    sValue = sValue.Substring(7).ToLower();
                if (sValue == "sbyte")
                    sValue = "byte";
                if (sValue == "uint16")
                    sValue = "int16";
                if (sValue == "uint32")
                    sValue = "int32";
                if (sValue == "uint64")
                    sValue = "int64";
                if (sValue == "timespan")
                    sValue = "double";

                switch (sValue)
                {
                    case "boolean":
                        oGridField.iFieldDataType = FieldType.Boolean;
                        if (m_oEntry.iLanguage == Language.CSharp)
                        {
                            oGridField.sFieldDataType = FieldTypeCSharp.Boolean;
                            oGridField.sFieldDefaultValue = FieldDefaultCSharp.Boolean;
                            oGridField.sFieldNameApp = FieldTypePrefixCSharp.Boolean + oGridField.sFieldName;
                        }
                        else
                        {
                            oGridField.sFieldDataType = FieldTypeVB.Boolean;
                            oGridField.sFieldDefaultValue = FieldDefaultVB.Boolean;
                            oGridField.sFieldNameApp = FieldTypePrefixVB.Boolean + oGridField.sFieldName;
                        }
                        break;

                    case "byte":
                        oGridField.iFieldDataType = FieldType.Byte;
                        if (m_oEntry.iLanguage == Language.CSharp)
                        {
                            oGridField.sFieldDataType = FieldTypeCSharp.Byte;
                            oGridField.sFieldDefaultValue = FieldDefaultCSharp.Byte;
                            oGridField.sFieldNameApp = FieldTypePrefixCSharp.Byte + oGridField.sFieldName;
                        }
                        else
                        {
                            oGridField.sFieldDataType = FieldTypeVB.Byte;
                            oGridField.sFieldDefaultValue = FieldDefaultVB.Byte;
                            oGridField.sFieldNameApp = FieldTypePrefixVB.Byte + oGridField.sFieldName;
                        }
                        break;

                    case "char":
                        oGridField.iFieldDataType = FieldType.Character;
                        if (m_oEntry.iLanguage == Language.CSharp)
                        {
                            oGridField.sFieldDataType = FieldTypeCSharp.Character;
                            oGridField.sFieldDefaultValue = FieldDefaultCSharp.Character;
                            oGridField.sFieldNameApp = FieldTypePrefixCSharp.Character + oGridField.sFieldName;
                        }
                        else
                        {
                            oGridField.sFieldDataType = FieldTypeVB.Character;
                            oGridField.sFieldDefaultValue = FieldDefaultVB.Character;
                            oGridField.sFieldNameApp = FieldTypePrefixVB.Character + oGridField.sFieldName;
                        }
                        break;

                    case "datetime":
                        oGridField.iFieldDataType = FieldType.DateTime;
                        if (m_oEntry.iLanguage == Language.CSharp)
                        {
                            oGridField.sFieldDataType = FieldTypeCSharp.DateTime;
                            oGridField.sFieldDefaultValue = FieldDefaultCSharp.DateTime;
                            oGridField.sFieldNameApp = FieldTypePrefixCSharp.DateTime + oGridField.sFieldName;
                        }
                        else
                        {
                            oGridField.sFieldDataType = FieldTypeVB.DateTime;
                            oGridField.sFieldDefaultValue = FieldDefaultVB.DateTime;
                            oGridField.sFieldNameApp = FieldTypePrefixVB.DateTime + oGridField.sFieldName;
                        }
                        break;

                    case "decimal":
                        oGridField.iFieldDataType = FieldType.Decimal;
                        if (m_oEntry.iLanguage == Language.CSharp)
                        {
                            oGridField.sFieldDataType = FieldTypeCSharp.Decimal;
                            oGridField.sFieldDefaultValue = FieldDefaultCSharp.Decimal;
                            oGridField.sFieldNameApp = FieldTypePrefixCSharp.Decimal + oGridField.sFieldName;
                        }
                        else
                        {
                            oGridField.sFieldDataType = FieldTypeVB.Decimal;
                            oGridField.sFieldDefaultValue = FieldDefaultVB.Decimal;
                            oGridField.sFieldNameApp = FieldTypePrefixVB.Decimal + oGridField.sFieldName;
                        }
                        break;

                    case "double":
                        oGridField.iFieldDataType = FieldType.Double;
                        if (m_oEntry.iLanguage == Language.CSharp)
                        {
                            oGridField.sFieldDataType = FieldTypeCSharp.Double;
                            oGridField.sFieldDefaultValue = FieldDefaultCSharp.Double;
                            oGridField.sFieldNameApp = FieldTypePrefixCSharp.Double + oGridField.sFieldName;
                        }
                        else
                        {
                            oGridField.sFieldDataType = FieldTypeVB.Double;
                            oGridField.sFieldDefaultValue = FieldDefaultVB.Double;
                            oGridField.sFieldNameApp = FieldTypePrefixVB.Double + oGridField.sFieldName;
                        }
                        break;

                    case "int16":
                        oGridField.iFieldDataType = FieldType.Short;
                        if (m_oEntry.iLanguage == Language.CSharp)
                        {
                            oGridField.sFieldDataType = FieldTypeCSharp.Short;
                            oGridField.sFieldDefaultValue = FieldDefaultCSharp.Short;
                            oGridField.sFieldNameApp = FieldTypePrefixCSharp.Short + oGridField.sFieldName;
                        }
                        else
                        {
                            oGridField.sFieldDataType = FieldTypeVB.Short;
                            oGridField.sFieldDefaultValue = FieldDefaultVB.Short;
                            oGridField.sFieldNameApp = FieldTypePrefixVB.Short + oGridField.sFieldName;
                        }
                        break;

                    case "int32":
                        oGridField.iFieldDataType = FieldType.Integer;
                        if (m_oEntry.iLanguage == Language.CSharp)
                        {
                            oGridField.sFieldDataType = FieldTypeCSharp.Integer;
                            oGridField.sFieldDefaultValue = FieldDefaultCSharp.Integer;
                            oGridField.sFieldNameApp = FieldTypePrefixCSharp.Integer + oGridField.sFieldName;
                        }
                        else
                        {
                            oGridField.sFieldDataType = FieldTypeVB.Integer;
                            oGridField.sFieldDefaultValue = FieldDefaultVB.Integer;
                            oGridField.sFieldNameApp = FieldTypePrefixVB.Integer + oGridField.sFieldName;
                        }
                        break;

                    case "int64":
                        oGridField.iFieldDataType = FieldType.Long;
                        if (m_oEntry.iLanguage == Language.CSharp)
                        {
                            oGridField.sFieldDataType = FieldTypeCSharp.Long;
                            oGridField.sFieldDefaultValue = FieldDefaultCSharp.Long;
                            oGridField.sFieldNameApp = FieldTypePrefixCSharp.Long + oGridField.sFieldName;
                        }
                        else
                        {
                            oGridField.sFieldDataType = FieldTypeVB.Long;
                            oGridField.sFieldDefaultValue = FieldDefaultVB.Long;
                            oGridField.sFieldNameApp = FieldTypePrefixVB.Long + oGridField.sFieldName;
                        }
                        break;

                    case "single":
                        oGridField.iFieldDataType = FieldType.Single;
                        if (m_oEntry.iLanguage == Language.CSharp)
                        {
                            oGridField.sFieldDataType = FieldTypeCSharp.Single;
                            oGridField.sFieldDefaultValue = FieldDefaultCSharp.Single;
                            oGridField.sFieldNameApp = FieldTypePrefixCSharp.Single + oGridField.sFieldName;
                        }
                        else
                        {
                            oGridField.sFieldDataType = FieldTypeVB.Single;
                            oGridField.sFieldDefaultValue = FieldDefaultVB.Single;
                            oGridField.sFieldNameApp = FieldTypePrefixVB.Single + oGridField.sFieldName;
                        }
                        break;

                    default:
                        oGridField.iFieldDataType = FieldType.String;
                        if (m_oEntry.iLanguage == Language.CSharp)
                        {
                            oGridField.sFieldDataType = FieldTypeCSharp.String;
                            oGridField.sFieldDefaultValue = FieldDefaultCSharp.String;
                            oGridField.sFieldNameApp = FieldTypePrefixCSharp.String + oGridField.sFieldName;
                        }
                        else
                        {
                            oGridField.sFieldDataType = FieldTypeVB.String;
                            oGridField.sFieldDefaultValue = FieldDefaultVB.String;
                            oGridField.sFieldNameApp = FieldTypePrefixVB.String + oGridField.sFieldName;
                        }
                        break;
                }
            }
            catch (Exception oEx)
            {
                m_oErrorMsgList.Add("An error occurred while determining a field type - "
                    + oEx.Message);
            }
        }

        private void ProcessEmit(
            string sLine,
            StringBuilder oBuffer)
        {
            try
            {
                string sResult = sLine;
                if (m_iObjectDataSourceIndex >= 0)
                {
                    sResult = sResult
                        .Replace("%ClassBeanName%", m_oObjectDataSourceList[m_iObjectDataSourceIndex].sObjectSourceBeanName)
                        .Replace("%ClassSourceName%", m_oObjectDataSourceList[m_iObjectDataSourceIndex].sObjectSourceName)
                        .Replace("%MethodDelete%", m_oObjectDataSourceList[m_iObjectDataSourceIndex].sMethodDelete)
                        .Replace("%MethodGet%", m_oObjectDataSourceList[m_iObjectDataSourceIndex].sMethodSelect)
                        .Replace("%MethodGetCount%", m_oObjectDataSourceList[m_iObjectDataSourceIndex].sMethodSelectCount)
                        .Replace("%MethodInsert%", m_oObjectDataSourceList[m_iObjectDataSourceIndex].sMethodInsert)
                        .Replace("%MethodUpdate%", m_oObjectDataSourceList[m_iObjectDataSourceIndex].sMethodUpdate)
                        .Replace("%SortParam%", m_oObjectDataSourceList[m_iObjectDataSourceIndex].sSortParam);
                }

                if (m_iFieldIndex >= 0 && m_iFieldIndex < m_oGridTableList[m_iGridIndex].oFields.Count)
                {
                    sResult = sResult
                        .Replace("%FieldDataType%", m_oGridTableList[m_iGridIndex].oFields[m_iFieldIndex].sFieldDataType)
                        .Replace("%FieldDefaultValue%", m_oGridTableList[m_iGridIndex].oFields[m_iFieldIndex].sFieldDefaultValue)
                        .Replace("%FieldName%", m_oGridTableList[m_iGridIndex].oFields[m_iFieldIndex].sFieldName)
                        .Replace("%FieldNameApp%", m_oGridTableList[m_iGridIndex].oFields[m_iFieldIndex].sFieldNameApp);
                }

                if (m_iCommandIndex >= 0 && m_iCommandIndex < m_oGridTableList[m_iGridIndex].oCommands.Count)
                {
                    sResult = sResult
                        .Replace("%CommandName%", m_oGridTableList[m_iGridIndex].oCommands[m_iCommandIndex].sCommandName);
                }
                oBuffer.Append(sResult);
                oBuffer.Append("\r\n");
            }
            catch (Exception oEx)
            {
                m_oErrorMsgList.Add("An error occurred while emitting a line - "
                    + oEx.Message);
            }
        }

        private bool ProcessFileExists(
            string sFileName,
            string sFileTitle)
        {
            if (!File.Exists(sFileName))
            {
                m_oErrorMsgList.Add("The " + sFileTitle + " file does not exist - " + sFileName);
                return false;
            }
            return true;
        }

        private bool ProcessIf(
            List<string> oLines,
            StringBuilder oBuffer,
            ref int iIndex,
            string sFileName,
            string sIfTag)
        {
            List<string> oLinesBlock = new List<string>();
            int iStartLine = iIndex;
            int iNestLevel = 0;
            string sLineText;

            try
            {
                while (true)
                {
                    iIndex++;
                    if (iIndex >= oLines.Count)
                    {
                        m_oErrorMsgList.Add("There was no terminator for a block If statement at line# "
                            + iStartLine
                            + " in file: "
                            + sFileName);
                        return false;
                    }

                    sLineText = oLines[iIndex].Trim().Replace("  ", " ").ToLower();
                    if (sLineText.StartsWith(sTagIf))
                        iNestLevel++;
                    if (sLineText.StartsWith(sTagEndIf))
                    {
                        if (iNestLevel <= 0)
                            break;
                        else
                            iNestLevel--;
                    }
                    oLinesBlock.Add(oLines[iIndex]);
                }

                bool bProcess = false;
                switch (sIfTag)
                {
                    case sTagIfDataReader:
                        bProcess =
                            (m_oEntry.iDataAccessObject == GridSourceEntry.DataAccessObject.DataReader) ? true : false;
                        break;
                    case sTagIfDataSet:
                        bProcess =
                            (m_oEntry.iDataAccessObject == GridSourceEntry.DataAccessObject.DataSet) ? true : false;
                        break;
                    case sTagIfDataTable:
                        bProcess =
                            (m_oEntry.iDataAccessObject == GridSourceEntry.DataAccessObject.DataTable) ? true : false;
                        break;
                    case sTagIfSortField:
                        bProcess =
                            m_oGridTableList[m_iGridIndex].oFields[m_iFieldIndex].bSortField;
                        break;
                    default:
                        m_oErrorMsgList.Add("An invalid operator for a block If statement was found at line# "
                            + iStartLine
                            + " - " + sIfTag
                            + "\r\nFile: "
                            + sFileName);
                        return false;
                }

                if (bProcess)
                {
                    for (int iLineIndex = 0; iLineIndex < oLinesBlock.Count; iLineIndex++)
                    {
                        sLineText = oLinesBlock[iLineIndex].Trim().ToLower();
                        if (sLineText.StartsWith(sTagControl))
                        {
                            string[] sArray = sLineText.Split(' ');
                            switch (sArray[0])
                            {
                                case sTagIf:
                                    if (!ProcessIf(
                                        oLinesBlock,
                                        oBuffer,
                                        ref iLineIndex,
                                        sFileName,
                                        sArray[1]))
                                        return false;
                                    break;

                                case sTagRepeat:
                                    if (!ProcessRepeat(
                                        oLines,
                                        oBuffer,
                                        ref iLineIndex,
                                        sFileName,
                                        sArray[1]))
                                        return false;
                                    break;

                                default:
                                    m_oErrorMsgList.Add("An invalid record control parameter ('"
                                        + sArray[0]
                                        + "') was found at line# "
                                        + iLineIndex
                                        + " in the file: "
                                        + sFileName);
                                    return false;
                            }
                        }
                        else
                        {
                            ProcessEmit(oLinesBlock[iLineIndex], oBuffer);
                        }
                    }
                }
                return true;
            }
            catch (Exception oEx)
            {
                m_oErrorMsgList.Add("An error occurred while processing a block If statement at line# "
                    + iIndex
                    + " in file: "
                    + sFileName
                    + " - "
                    + oEx.Message);
                return false;
            }
        }

        private StringBuilder ProcessImport(
            string sFileName,
            string sRoutineTitle)
        {
            try
            {
                StringBuilder oBuffer = new StringBuilder();
                string[] sLines = null;

                sLines = File.ReadAllLines(sFileName);
                List<string> oLines = new List<string>(sLines);
                bool bEmitLine = true;

                for (int iLineIndex = 0; iLineIndex < oLines.Count; iLineIndex++)
                {
                    string sLineText = oLines[iLineIndex].Trim().ToLower().Replace("  ", " ");
                    if (sLineText.StartsWith(sTagControl))
                    {
                        string[] sArray = sLineText.Split(' ');
                        switch (sArray[0])
                        {
                            case sTagIf:
                                if (bEmitLine)
                                {
                                    if (!ProcessIf(
                                        oLines,
                                        oBuffer,
                                        ref iLineIndex,
                                        sFileName,
                                        sArray[1]))
                                        return null;
                                }
                                break;

                            case sTagRepeat:
                                if (bEmitLine)
                                {
                                    if (!ProcessRepeat(
                                        oLines,
                                        oBuffer,
                                        ref iLineIndex,
                                        sFileName,
                                        sArray[1]))
                                        return null;
                                }
                                break;

                            case sTagInsert:
                                switch (sArray[1])
                                {
                                    case sTagInsertImports:
                                        bEmitLine = ProcessImportInsert(m_oCustomImports, oBuffer);
                                        break;
                                    case sTagInsertConstants:
                                        bEmitLine = ProcessImportInsert(m_oCustomConstants, oBuffer);
                                        break;
                                    case sTagInsertData:
                                        bEmitLine = ProcessImportInsert(m_oCustomData, oBuffer);
                                        break;
                                    case sTagInsertConstructors:
                                        bEmitLine = ProcessImportInsert(m_oCustomConstructors, oBuffer);
                                        break;
                                    case sTagInsertProperties:
                                        bEmitLine = ProcessImportInsert(m_oCustomProperties, oBuffer);
                                        break;
                                    case sTagInsertRoutines:
                                        bEmitLine = ProcessImportInsert(m_oCustomRoutines, oBuffer);
                                        break;
                                }
                                break;

                            case sTagEndInsert:
                                bEmitLine = true;
                                break;

                            default:
                                if (bEmitLine)
                                {
                                    m_oErrorMsgList.Add("An invalid record control parameter ('"
                                        + sArray[0]
                                        + "') was found at line# "
                                        + iLineIndex
                                        + " in the file "
                                        + sFileName);
                                    return null;
                                }
                                break;
                        }
                    }

                    else
                    {
                        if (bEmitLine)
                            ProcessEmit(oLines[iLineIndex], oBuffer);
                    }
                }

                return oBuffer;
            }
            catch (Exception oEx)
            {
                m_oErrorMsgList.Add("An error occurred while generating " + sRoutineTitle + " code - "
                    + oEx.Message);
                return null;
            }
        }

        private bool ProcessImportInsert(List<string> oInsert, StringBuilder oBuffer)
        {
            if (oInsert == null || oInsert.Count <= 0)
                return true;
            else
            {
                foreach (string sLine in oInsert)
                {
                    oBuffer.Append(sLine);
                    oBuffer.Append("\r\n");
                }
            }
            return false;
        }

        private bool ProcessRepeat(
            List<string> oLines,
            StringBuilder oBuffer,
            ref int iIndex,
            string sFileName,
            string sRepeatTag)
        {
            List<string> oLinesBlock = new List<string>();
            int iStartLine = iIndex;
            string sLineText;

            try
            {
                while (true)
                {
                    iIndex++;
                    if (iIndex >= oLines.Count)
                    {
                        m_oErrorMsgList.Add("There was no terminator for a block Repeat statement at line# "
                            + iStartLine
                            + " in file: "
                            + sFileName);
                        return false;
                    }

                    sLineText = oLines[iIndex].Trim().Replace("  ", " ").ToLower();
                    if (sLineText.StartsWith(sTagEndRepeat))
                        break;
                    oLinesBlock.Add(oLines[iIndex]);
                }

                switch (sRepeatTag.ToLower())
                {
                    case "field":
                        for (m_iFieldIndex = 0; m_iFieldIndex < m_oGridTableList[m_iGridIndex].oFields.Count ; m_iFieldIndex++)
                        {
                            for (int iBlockIndex = 0; iBlockIndex < oLinesBlock.Count; iBlockIndex++)
                            {
                                sLineText = oLinesBlock[iBlockIndex].Trim().ToLower();
                                if (sLineText.StartsWith(sTagControl))
                                {
                                    string[] sArray = sLineText.Split(' ');
                                    if (sArray[0] == sTagIf)
                                    {
                                        if (!ProcessIf(
                                            oLinesBlock,
                                            oBuffer,
                                            ref iBlockIndex,
                                            sFileName,
                                            sArray[1]))
                                            return false;
                                    }
                                    else
                                    {
                                        m_oErrorMsgList.Add("An invalid record control parameter ('"
                                            + sArray[0]
                                            + "') was found at line# "
                                            + iBlockIndex
                                            + " in the file: "
                                            + sFileName);
                                        return false;
                                    }
                                }
                                else
                                {
                                    ProcessEmit(oLinesBlock[iBlockIndex], oBuffer);
                                }
                            }
                        }
                        break;

                    case "command":
                        for (m_iCommandIndex = 0; m_iCommandIndex < m_oGridTableList[m_iGridIndex].oCommands.Count; m_iCommandIndex++)
                        {
                            for (int iBlockIndex = 0; iBlockIndex < oLinesBlock.Count; iBlockIndex++)
                            {
                                sLineText = oLinesBlock[iBlockIndex].Trim().ToLower();
                                if (sLineText.StartsWith(sTagControl))
                                {
                                    string[] sArray = sLineText.Split(' ');
                                    if (sArray[0] == sTagIf)
                                    {
                                        if (!ProcessIf(
                                            oLinesBlock,
                                            oBuffer,
                                            ref iBlockIndex,
                                            sFileName,
                                            sArray[1]))
                                            return false;
                                    }
                                    else
                                    {
                                        m_oErrorMsgList.Add("An invalid record control parameter ('"
                                            + sArray[0]
                                            + "') was found at line# "
                                            + iBlockIndex
                                            + " in the file: "
                                            + sFileName);
                                        return false;
                                    }
                                }
                                else
                                {
                                    ProcessEmit(oLinesBlock[iBlockIndex], oBuffer);
                                }
                            }
                        }
                        break;
                }
                return true;
            }
            catch (Exception oEx)
            {
                m_oErrorMsgList.Add("An error occurred while processing a block Repeat statement at line# "
                    + iIndex
                    + " in file name: "
                    + sFileName
                    + " - "
                    + oEx.Message);
                return false;
            }
        }
        #endregion
    }
}
