﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using UtilityCommonDialoger;

namespace GridSourceCreator
{
    public partial class frmEntry : Form
    {
        #region Constants
        private const string sAppTitle = "Grid Source Creator";
        #endregion

        #region Data
        private GenerateContent.GridSourceEntry m_oEntry = null;
        private List<GenerateContent.GridSourceEntry> m_oEntryList = null;
        private string sProjectPath = "";
        #endregion

        #region Form Routines
        public frmEntry(
            ref GenerateContent.GridSourceEntry oEntry,
            ref List<GenerateContent.GridSourceEntry> oEntryList,
            string sEntryDescription)
        {
            InitializeComponent();
            m_oEntry = oEntry;
            m_oEntryList = oEntryList;
            LoadData();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            try
            {
                lblProjectSolutionFileName.Text = lblProjectSolutionFileName.Text.Trim();
                if (String.IsNullOrEmpty(lblProjectSolutionFileName.Text))
                {
                    MessageBox.Show(
                        "Select the project soulution file for this grid source",
                        sAppTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Warning);
                    btnSelectProject.Focus();
                    return;
                }

                lblProjectASPXFileName.Text = lblProjectASPXFileName.Text.Trim();
                if (String.IsNullOrEmpty(lblProjectASPXFileName.Text))
                {
                    MessageBox.Show(
                        "Select the ASPX page file for this grid source",
                        sAppTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Warning);
                    btnSelectFile.Focus();
                    return;
                }

                lblProjectClassPath.Text = lblProjectClassPath.Text.Trim();
                if (String.IsNullOrEmpty(lblProjectClassPath.Text))
                {
                    MessageBox.Show(
                        "Select an output path for the grid source files",
                        sAppTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Warning);
                    btnSetPath.Focus();
                    return;
                }

                lblDescription.Text = lblDescription.Text.Trim();
                if (String.IsNullOrEmpty(lblDescription.Text))
                {
                    MessageBox.Show(
                        "The description for this grid source must be set - re-select the solution and ASPX files",
                        sAppTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Warning);
                    btnSelectProject.Focus();
                    return;
                }

                m_oEntry.sDescription = lblDescription.Text;
                m_oEntry.sProjectSolutionFileName = lblProjectSolutionFileName.Text;
                m_oEntry.sProjectASPXFileName = lblProjectASPXFileName.Text;
                m_oEntry.sProjectClassPath = lblProjectClassPath.Text;
                if (optDataTable.Checked)
                    m_oEntry.iDataAccessObject = GenerateContent.GridSourceEntry.DataAccessObject.DataTable;
                else if (optDataSet.Checked)
                    m_oEntry.iDataAccessObject = GenerateContent.GridSourceEntry.DataAccessObject.DataSet;
                else
                    m_oEntry.iDataAccessObject = GenerateContent.GridSourceEntry.DataAccessObject.DataReader;
                if (optCSharp.Checked)
                    m_oEntry.iLanguage = GenerateContent.Language.CSharp;
                else
                    m_oEntry.iLanguage = GenerateContent.Language.VB;
                m_oEntry.bUpdate = true;
                this.Close();
            }
            catch (Exception oEx)
            {
                MessageBox.Show(
                    "An error occurred while processing the Database Entry - " + oEx.Message,
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Stop);
            }
        }

        private void btnSelectFile_Click(object sender, EventArgs e)
        {
            string sFileName = CommonDialoger.FileOpen(
                ref ctlOpenFileDialog,
                "",
                sProjectPath,
                "",
                "aspx|*.aspx",
                true,
                true,
                true,
                false);
            if (!String.IsNullOrEmpty(sFileName))
            {
                lblProjectASPXFileName.Text = sFileName;
                SetDescription();
            }
        }

        private void btnSetPath_Click(object sender, EventArgs e)
        {
            string sPath = CommonDialoger.FolderBrowser(
                ref ctlFolderBrowserDialog,
                sProjectPath,
                true);
            if (!String.IsNullOrEmpty(sPath))
                lblProjectClassPath.Text = sPath;
        }

        private void btnSelectProject_Click(object sender, EventArgs e)
        {
            string sFileName = CommonDialoger.FileOpen(
                ref ctlOpenFileDialog,
                "",
                "",
                "",
                "sln|*.sln",
                true,
                true,
                true,
                false);
            if (!String.IsNullOrEmpty(sFileName))
            {
                lblProjectSolutionFileName.Text = sFileName;
                sProjectPath = Path.GetDirectoryName(lblProjectSolutionFileName.Text);
                SetDescription();
            }
        }
        #endregion

        #region Private Routines
        private void LoadData()
        {
            lblDescription.Text = m_oEntry.sDescription;
            lblProjectSolutionFileName.Text = m_oEntry.sProjectSolutionFileName;
            lblProjectASPXFileName.Text = m_oEntry.sProjectASPXFileName;
            lblProjectClassPath.Text = m_oEntry.sProjectClassPath;
            switch (m_oEntry.iDataAccessObject)
            {
                case GenerateContent.GridSourceEntry.DataAccessObject.DataSet:
                    optDataSet.Checked = true;
                    break;
                case GenerateContent.GridSourceEntry.DataAccessObject.DataTable:
                    optDataTable.Checked = true;
                    break;
                default:
                    optDataReader.Checked = true;
                    break;
            }
            switch (m_oEntry.iLanguage)
            {
                case GenerateContent.Language.CSharp:
                    optCSharp.Checked = true;
                    break;
                default:
                    optVB.Checked = true;
                    break;
            }
        }

        private void SetDescription()
        {
            if (!String.IsNullOrEmpty(lblProjectSolutionFileName.Text) && !String.IsNullOrEmpty(lblProjectASPXFileName.Text))
            {
                lblDescription.Text =
                    Path.GetFileNameWithoutExtension(lblProjectSolutionFileName.Text) +
                    "/" +
                    Path.GetFileNameWithoutExtension(lblProjectASPXFileName.Text);
            }
        }
        #endregion
    }
}
