﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Windows.Forms;
using UtilityINI;

namespace GridSourceCreator
{
    public partial class frmMain : Form
    {
        #region Constants
        private const string sAppTitle = "Grid Source Creator";
        private const string sINIFileName = "GridSourceCreator.ini";
        private const string sINISectionName = "GridSourceCreator";
        private const string sLogFileName = "GridSourceCreator.log";
        #endregion

        #region Data
        private GenerateContent.GridSourceEntry m_oEntry = null;
        private List<GenerateContent.GridSourceEntry> m_oEntryList = null;
        private string m_sEntryDescription = "";
        private string m_sINIFIleName = "";
        private string m_sPathApp = "";
        #endregion

        #region Form Routines
        public frmMain()
        {
            InitializeComponent();
            m_sPathApp = Application.StartupPath + "\\";
            m_sINIFIleName = m_sPathApp + sINIFileName;
            if (!File.Exists(m_sINIFIleName))
                File.WriteAllText(m_sINIFIleName, "[" + sINISectionName + "]\r\n", Encoding.Default);
            LoadData();
            LoadList();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void btnDeleteEntry_Click(object sender, EventArgs e)
        {
            if (cboEntry.Items.Count <= 0)
            {
                MessageBox.Show(
                    "There are no entries in the list",
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Warning);
                return;
            }
            if (cboEntry.SelectedIndex < 0)
            {
                MessageBox.Show(
                    "Select a grid source entry from the list",
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Warning);
                return;
            }
            if (MessageBox.Show(
                "Are you sure that you want to delete this grid source entry?",
                    sAppTitle,
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question) == DialogResult.Yes)
            {
                DeleteData(m_oEntryList[cboEntry.SelectedIndex].sDescription);
                LoadList();
                WriteHeader();
            }
        }

        private void btnEditEntry_Click(object sender, EventArgs e)
        {
            if (cboEntry.Items.Count <= 0)
            {
                MessageBox.Show(
                    "There are no entries in the list",
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Warning);
                return;
            }
            if (cboEntry.SelectedIndex < 0)
            {
                MessageBox.Show(
                    "Select a grid source entry from the list",
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Warning);
                return;
            }

            m_oEntry = m_oEntryList[cboEntry.SelectedIndex];
            ShowForm();
        }

        private void btnGenerate_Click(object sender, EventArgs e)
        {
            if (cboEntry.Items.Count <= 0)
            {
                MessageBox.Show(
                    "There are no entries in the list",
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Warning);
                return;
            }
            if (cboEntry.SelectedIndex < 0)
            {
                MessageBox.Show(
                    "Select a database entry from the list",
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Warning);
                return;
            }

            int iGenerateCount;

            Cursor.Current = Cursors.WaitCursor;
            m_oEntry = m_oEntryList[cboEntry.SelectedIndex];
            GenerateContent oGenerateContent = new GenerateContent(
                m_sPathApp,
                m_oEntry);
            List<string> oErrorMsg = oGenerateContent.Generate(out iGenerateCount);
            Cursor.Current = Cursors.Default;
            if (oErrorMsg.Count <= 0)
            {
                if (iGenerateCount <= 0)
                {
                    MessageBox.Show(
                        "No files were generated",
                        sAppTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show(
                        "The grid source files were successfully generated",
                        sAppTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Information);
                }
            }
            else
            {
                MessageBox.Show(
                    "The generation process completed with errors - the log will be automatically displayed",
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Stop);

                string sFileName = m_sPathApp + sLogFileName;
                File.Delete(sFileName);
                foreach (string sLine in oErrorMsg)
                {
                    File.AppendAllText(sFileName, sLine + "\r\n");
                }
                Process oNotepad = new Process();
                oNotepad.StartInfo.FileName = "notepad.exe";
                oNotepad.StartInfo.Arguments = sFileName;
                oNotepad.Start();
            }
        }

        private void btnNewEntry_Click(object sender, EventArgs e)
        {
            m_oEntry = new GenerateContent.GridSourceEntry();
            ShowForm();
        }
        #endregion

        #region Private Routines
        private void DeleteData(string sDescription)
        {
            INI.DeleteSection(m_sINIFIleName, sDescription);
            m_oEntryList.RemoveAt(cboEntry.SelectedIndex);
        }

        private void LoadData()
        {
            try
            {
                m_oEntryList = new List<GenerateContent.GridSourceEntry>();
                string sGridSourceEntries = INI.GetString(m_sINIFIleName, sINISectionName, "Entries", "").Trim();
                if (!String.IsNullOrEmpty(sGridSourceEntries))
                {
                    // Read in the database entries from the INI file
                    string[] sArray = sGridSourceEntries.Split(',');
                    foreach (string sEntry in sArray)
                    {
                        GenerateContent.GridSourceEntry oGridSourceEntry = new GenerateContent.GridSourceEntry();
                        string sSectionName = sEntry.Trim();
                        oGridSourceEntry.sDescription = sEntry.Trim();
                        oGridSourceEntry.sProjectSolutionFileName = INI.GetString(m_sINIFIleName, sSectionName, "ProjectSolutionFileName", "").Trim();
                        oGridSourceEntry.sProjectASPXFileName = INI.GetString(m_sINIFIleName, sSectionName, "ProjectASPXFileName", "").Trim();
                        oGridSourceEntry.sProjectClassPath = INI.GetString(m_sINIFIleName, sSectionName, "ProjectClassPath", "").Trim();
                        oGridSourceEntry.iDataAccessObject = (GenerateContent.GridSourceEntry.DataAccessObject)INI.GetInteger(m_sINIFIleName, sSectionName, "DataAccessObject", oGridSourceEntry.iDataAccessObject.ToString());
                        oGridSourceEntry.iLanguage = (GenerateContent.Language)INI.GetInteger(m_sINIFIleName, sSectionName, "Language", oGridSourceEntry.iLanguage.ToString());
                        m_oEntryList.Add(oGridSourceEntry);
                    }
                }
            }
            catch (Exception oEx)
            {
                MessageBox.Show(
                    "An error occurred while loading the INI file data - " + oEx.Message,
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Stop);
            }
        }

        /// <summary>
        /// Insert the list of database entries into the combox box
        /// </summary>
        private void LoadList()
        {
            try
            {
                cboEntry.Items.Clear();

                if (m_oEntryList.Count <= 0)
                {
                    btnDeleteEntry.Enabled = false;
                    btnEditEntry.Enabled = false;
                    btnGenerate.Enabled = false;
                }
                else
                {
                    btnDeleteEntry.Enabled = true;
                    btnEditEntry.Enabled = true;
                    btnGenerate.Enabled = true;
                    // Sort the list
                    m_oEntryList.Sort(delegate(GenerateContent.GridSourceEntry o1, GenerateContent.GridSourceEntry o2)
                    {
                        return o1.sDescription.CompareTo(o2.sDescription);
                    });

                    // Insert the sorted list into the drop-down list box
                    for (int iX = 0; iX < m_oEntryList.Count; iX++)
                    {
                        cboEntry.Items.Add(m_oEntryList[iX].sDescription);
                    }
                    if (cboEntry.Items.Count > 0)
                        cboEntry.SelectedIndex = 0;
                }
            }
            catch (Exception oEx)
            {
                MessageBox.Show(
                    "An error occurred while loading the combo box - " + oEx.Message,
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Stop);
            }
        }

        private void ShowForm()
        {
            m_sEntryDescription = m_oEntry.sDescription;
            m_oEntry.bUpdate = false;
            frmEntry ofrmDatabase = new frmEntry(
                ref m_oEntry,
                ref m_oEntryList,
                m_sEntryDescription);
            ofrmDatabase.ShowDialog();
            if (m_oEntry.bUpdate)
                UpdateData();
        }

        private void UpdateData()
        {
            try
            {
                // Determine if the entry is already in the list
                if (String.IsNullOrEmpty(m_sEntryDescription))
                {
                    // Add the new entry
                    m_oEntryList.Add(m_oEntry);
                }
                else
                {
                    if (m_sEntryDescription != m_oEntry.sDescription)
                    {
                        DeleteData(m_sEntryDescription);
                        m_oEntryList.Add(m_oEntry);
                    }
                    else
                        m_oEntryList[cboEntry.SelectedIndex] = m_oEntry;
                }

                LoadList();
                WriteHeader();
                string sSectionName = m_oEntry.sDescription;
                INI.SetString(m_sINIFIleName, sSectionName, "ProjectSolutionFileName", m_oEntry.sProjectSolutionFileName);
                INI.SetString(m_sINIFIleName, sSectionName, "ProjectASPXFileName", m_oEntry.sProjectASPXFileName);
                INI.SetString(m_sINIFIleName, sSectionName, "ProjectClassPath", m_oEntry.sProjectClassPath);
                INI.SetString(m_sINIFIleName, sSectionName, "DataAccessObject", Convert.ToInt32(m_oEntry.iDataAccessObject).ToString());
                INI.SetString(m_sINIFIleName, sSectionName, "Language", Convert.ToInt32(m_oEntry.iLanguage).ToString());
                INI.SetString(m_sINIFIleName, sSectionName, "Separator", new string('-', 100));
            }
            catch (Exception oEx)
            {
                MessageBox.Show(
                    "An error occurred while loading the combo box - " + oEx.Message,
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Stop);
            }
        }

        private void WriteHeader()
        {
            try
            {
                string sEntry = "";
                for (int iX = 0; iX < m_oEntryList.Count; iX++)
                {
                    sEntry += ((String.IsNullOrEmpty(sEntry)) ? "" : ",")
                        + m_oEntryList[iX].sDescription;
                }
                INI.SetString(m_sINIFIleName, sINISectionName, "Entries", sEntry);
                INI.SetString(m_sINIFIleName, sINISectionName, "Separator", new string('-', 100));
            }
            catch (Exception oEx)
            {
                MessageBox.Show(
                    "An error occurred while loading the combo box - " + oEx.Message,
                    sAppTitle,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Stop);
            }
        }
        #endregion
    }
}
